import { IPubSub } from "../tupelo";
import { EcdsaKey } from "../crypto";
/**
 * CommunityMessenger handles the overlay pubsub community provides to make sure
 * that nodes correctly relay messages within this community.
 * Community listends to a number of libp2p topics (we call shards here) and internal community topics
 * are broadcast on those libp2p topics so that libp2p will correctly relay those messages.
 * @beta
 */
export declare class CommunityMessenger {
    name: string;
    localIdentifier: Uint8Array;
    private key;
    private shards;
    private pubsub;
    private subscriptions;
    constructor(name: string, shards: number, key: EcdsaKey, localIdentifier: Uint8Array, pubsub: IPubSub);
    /**
     *
     * @param topic - the topic to broadcast to (the community topic, not the pubsub topic)
     * @param payload - the data to send
     * @beta
     */
    publish(topic: string, payload: Uint8Array): Promise<unknown>;
    /**
     *
     * @param topic - the community topic to subscribe to
     * @param subscriber - a callback function which will be called with a protobuf instance of Envelope ( the js protobuf object, not the bytes )
     * @beta
     */
    subscribe(topic: string, subscriber: Function): Promise<unknown>;
    /**
     * unsubscribe is the reverse of subscribe and works the same as libp2p pubsub works,
     * you pass in a topic and the original subscription and your callback will no longer
     * receive messages from that topic
     * @param topic - the community topic used to subscribe
     * @param subscriber - the callback function your originally passed to subscribe
     * @beta
     */
    unsubscribe(topic: string, subscriber: Function): Promise<unknown>;
    private _communityTopic;
    private indexOfSubscriber;
}
export default CommunityMessenger;
//# sourceMappingURL=messaging.d.ts.map