import { IBlock } from '../chaintree/dag/dag';
import CID from 'cids';
/**
 * The callback-based version of Bitswap (current master).
 * @public
 */
export interface ICallbackBitswap {
    put(block: IBlock, cb: Function): void;
    get(cid: CID, cb: Function): void;
    start(cb: Function): void;
    stop(cb: Function): void;
    wantlistForPeer(peerID: any): any;
    ledgerForPeer(peerId: any): Object;
    getMany(cids: CID[], callback: Function): void;
    putMany(blocks: IBlock[], callback: Function): void;
    getWantlist(): any;
    peers(): any;
    stat(): any;
}
/**
 * A wrapper around an IPFS call-back-based bitswap that turns it into async/await compatible.
 * There is currently an open PR on the bitswap repo that will allow us to switch to the IPFS-maintained bitswap completely.
 * @public
 */
export declare class WrappedBitswap {
    private bitswap;
    constructor(bitswap: ICallbackBitswap);
    get(cid: CID): Promise<IBlock>;
    put(block: IBlock): Promise<any>;
    delete(cid: CID): Promise<unknown>;
    start(): () => Promise<unknown>;
    stop(): () => Promise<unknown>;
    wantlistForPeer(peerID: any): any;
    ledgerForPeer(peerID: any): Object;
    getMany(cids: CID[]): Promise<unknown>;
    putMany(blocks: IBlock[]): Promise<unknown>;
    getWantlist(): any;
    peers(): any;
    stat(): any;
}
//# sourceMappingURL=wrappedbitswap.d.ts.map