"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
const debug_1 = __importDefault(require("debug"));
const log = debug_1.default("wrappedbitswap");
/**
 * A wrapper around an IPFS call-back-based bitswap that turns it into async/await compatible.
 * There is currently an open PR on the bitswap repo that will allow us to switch to the IPFS-maintained bitswap completely.
 * @public
 */
class WrappedBitswap {
    constructor(bitswap) {
        this.bitswap = bitswap;
    }
    get(cid) {
        return util_1.default.promisify(this.bitswap.get.bind(this.bitswap))(cid);
    }
    put(block) {
        let resolve, reject;
        const p = new Promise((res, rej) => { resolve = res, reject = rej; });
        log("bitswap put: ", this.bitswap.put.toString());
        this.bitswap.put(block, (err) => {
            if (err) {
                reject(err);
            }
            resolve();
        });
        return p;
        // return util.promisify(this.bitswap.put.bind(this.bitswap))(block) as Promise<any>
    }
    delete(cid) {
        return new Promise((_, reject) => { reject(new Error("can't delete from bitswap")); });
    }
    start() {
        return util_1.default.promisify(this.bitswap.start.bind(this.bitswap));
    }
    stop() {
        return util_1.default.promisify(this.bitswap.stop.bind(this.bitswap));
    }
    wantlistForPeer(peerID) {
        return this.bitswap.wantlistForPeer(peerID);
    }
    ledgerForPeer(peerID) {
        return this.bitswap.ledgerForPeer(peerID);
    }
    getMany(cids) {
        return util_1.default.promisify(this.bitswap.getMany.bind(this.bitswap))(cids);
    }
    putMany(blocks) {
        return util_1.default.promisify(this.bitswap.putMany.bind(this.bitswap))(blocks);
    }
    getWantlist() {
        return this.bitswap.getWantlist();
    }
    peers() {
        return this.bitswap.peers();
    }
    stat() {
        return this.bitswap.stat();
    }
}
exports.WrappedBitswap = WrappedBitswap;
