"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
const IpfsRepo = require('ipfs-repo');
/**
 * Repo is a typescript wrapper around the IPFS Repo {@link https://github.com/ipfs/js-ipfs-repo}
 * @public
 */
class Repo {
    /**
     * @param name - The name of the repo
     * @param opts - (optional) {@link RepoOpts} - if opts are unspecified, it will use the IPFS repo defaults
     * @public
     */
    constructor(name, opts) {
        this.repo = new IpfsRepo(name, opts);
    }
    init(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            return util_1.default.promisify(this.repo.init.bind(this.repo))(opts);
        });
    }
    open() {
        return __awaiter(this, void 0, void 0, function* () {
            return util_1.default.promisify(this.repo.open.bind(this.repo))();
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            return util_1.default.promisify(this.repo.close.bind(this.repo))();
        });
    }
    put(key, val) {
        return __awaiter(this, void 0, void 0, function* () {
            return util_1.default.promisify(this.repo.datastore.put.bind(this.repo.datastore))(key, val);
        });
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return util_1.default.promisify(this.repo.datastore.get.bind(this.repo.datastore))(key);
        });
    }
    query(query) {
        return this.repo.datastore.query(query);
    }
}
exports.Repo = Repo;
exports.default = Repo;
