"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cids_1 = __importDefault(require("cids"));
const go = require('./js/go');
const transactions_pb_1 = require("tupelo-messages/transactions/transactions_pb");
const signatures_pb_1 = require("tupelo-messages/signatures/signatures_pb");
const debug_1 = __importDefault(require("debug"));
const logger = debug_1.default("tupelo");
class UnderlyingWasm {
    generateKey() {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    passPhraseKey(phrase, salt) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    keyFromPrivateBytes(bytes) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    ecdsaPubkeyToDid(pubKey) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    ecdsaPubkeyToAddress(pubKey) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    getCurrentState(opts) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    newEmptyTree(store, publicKey) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    playTransactions(opts) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    tokenPayloadForTransaction(opts) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    hashToShardNumber(topic, numberOfShards) {
        return 0; // replaced by wasm
    }
    getSendableEnvelopeBytes(envelopeBytes, key) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    verifyCurrentState(notaryGroupBytes, currStateBytes) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    signMessage(privateKey, message) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
    verifyMessage(addr, message, sigBits) {
        return new Promise((res, rej) => { }); // replaced by wasm
    }
}
var TupeloWasm;
(function (TupeloWasm) {
    let _tupelowasm;
    TupeloWasm.get = () => {
        if (_tupelowasm !== undefined) {
            return _tupelowasm;
        }
        _tupelowasm = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            const wasm = new UnderlyingWasm;
            logger("go.run for first time");
            go.run("./main.wasm");
            yield go.ready();
            go.populate(wasm, {
                "cids": cids_1.default,
                "ipfs-block": require('ipfs-block'),
            });
            resolve(wasm);
        }));
        return _tupelowasm;
    };
})(TupeloWasm || (TupeloWasm = {}));
/**
 * Tupelo is the more "raw" namespace, it is generally expected
 * that you would use higher level wrapper classes around this namespace.
 * For example: {@link EcdsaKey} or {@link Community#"class"}
 * @public
 */
var Tupelo;
(function (Tupelo) {
    // generateKey returns a two element array of the bytes for [privateKey, publicKey]
    function generateKey() {
        return __awaiter(this, void 0, void 0, function* () {
            logger("generateKey");
            const tw = yield TupeloWasm.get();
            return tw.generateKey();
        });
    }
    Tupelo.generateKey = generateKey;
    function passPhraseKey(phrase, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("passPhraseKey");
            const tw = yield TupeloWasm.get();
            return tw.passPhraseKey(phrase, salt);
        });
    }
    Tupelo.passPhraseKey = passPhraseKey;
    function keyFromPrivateBytes(bytes) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("keyFromPrivateBytes");
            const tw = yield TupeloWasm.get();
            return tw.keyFromPrivateBytes(bytes);
        });
    }
    Tupelo.keyFromPrivateBytes = keyFromPrivateBytes;
    function ecdsaPubkeyToDid(pubKey) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("ecdsaPubkeyToDid");
            const tw = yield TupeloWasm.get();
            return tw.ecdsaPubkeyToDid(pubKey);
        });
    }
    Tupelo.ecdsaPubkeyToDid = ecdsaPubkeyToDid;
    function ecdsaPubkeyToAddress(pubKey) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("ecdsaPubkeyToAddress");
            const tw = yield TupeloWasm.get();
            return tw.ecdsaPubkeyToAddress(pubKey);
        });
    }
    Tupelo.ecdsaPubkeyToAddress = ecdsaPubkeyToAddress;
    function getCurrentState(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("getCurrentState");
            const tw = yield TupeloWasm.get();
            try {
                let stateBits = yield tw.getCurrentState(opts);
                return signatures_pb_1.TreeState.deserializeBinary(stateBits);
            }
            catch (err) {
                throw err;
            }
        });
    }
    Tupelo.getCurrentState = getCurrentState;
    // newEmptyTree creates a new ChainTree with the ID populateed in the IBlockService and
    // returns the CID to the tip
    function newEmptyTree(store, publicKey) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("newEmptyTree");
            const tw = yield TupeloWasm.get();
            return tw.newEmptyTree(store, publicKey);
        });
    }
    Tupelo.newEmptyTree = newEmptyTree;
    /**
     * signMessage allows an EcdsaKey to sign an arbitrary message
     * @param key - the EcdsaKey to sign the message (must have a privateKey)
     * @param message - the message to sign (arbitrary Uint8Arry/Buffer of bytes)
     * @public
     */
    function signMessage(key, message) {
        return __awaiter(this, void 0, void 0, function* () {
            if (key.privateKey === undefined) {
                throw new Error("key must contain a privat key to sign messages");
            }
            const tw = yield TupeloWasm.get();
            try {
                const sigBits = yield tw.signMessage(key.privateKey, message);
                return signatures_pb_1.Signature.deserializeBinary(sigBits);
            }
            catch (e) {
                throw e;
            }
        });
    }
    Tupelo.signMessage = signMessage;
    /**
     * verifyMessage is the recipricol of {@link signMessage}, it verifies that a signature is valid.
     * @param address - the string address to verify against ( see {@link ecdsaPubkeyToAddress} )
     * @param message - the message to verify ( see {@link signMessage} )
     * @param signature - the {@link Signature} object to verify
     * @public
     */
    function verifyMessage(address, message, signature) {
        return __awaiter(this, void 0, void 0, function* () {
            const tw = yield TupeloWasm.get();
            try {
                return yield tw.verifyMessage(address, message, signature.serializeBinary());
            }
            catch (e) {
                throw e;
            }
        });
    }
    Tupelo.verifyMessage = verifyMessage;
    function tokenPayloadForTransaction(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("tokenPayloadForTransaction");
            const tw = yield TupeloWasm.get();
            const respBits = yield tw.tokenPayloadForTransaction({
                blockService: opts.blockService,
                tip: opts.tip,
                tokenName: opts.tokenName,
                sendId: opts.sendId,
                jsSendTxState: opts.treeState.serializeBinary(),
            });
            return transactions_pb_1.TokenPayload.deserializeBinary(respBits);
        });
    }
    Tupelo.tokenPayloadForTransaction = tokenPayloadForTransaction;
    function hashToShardNumber(topic, maxShards) {
        return __awaiter(this, void 0, void 0, function* () {
            const tw = yield TupeloWasm.get();
            return tw.hashToShardNumber(topic, maxShards);
        });
    }
    Tupelo.hashToShardNumber = hashToShardNumber;
    function getSendableEnvelopeBytes(env, key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (key.privateKey === undefined) {
                throw new Error("key needs to have a private key in order to sign the envelope");
            }
            const tw = yield TupeloWasm.get();
            const envBits = env.serializeBinary();
            const keyBits = key.privateKey;
            logger("getSendableEnvelopeBytes to wasm");
            return tw.getSendableEnvelopeBytes(envBits, keyBits);
        });
    }
    Tupelo.getSendableEnvelopeBytes = getSendableEnvelopeBytes;
    /**
     * verifyCurrentState takes a notary group and a currentstate and makes sure it meets the requirements
     * for being a valid signed currentState (2/3 of signers, valid signature, etc)
     * @param notaryGroup - the protobuf version of the notary group config
     * @param state - the CurrentState (often returned by a playTransactions)
     * @public
     */
    function verifyCurrentState(notaryGroup, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const tw = yield TupeloWasm.get();
            return tw.verifyCurrentState(notaryGroup.serializeBinary(), state.serializeBinary());
        });
    }
    Tupelo.verifyCurrentState = verifyCurrentState;
    function playTransactions(publisher, notaryGroup, tree, transactions) {
        return __awaiter(this, void 0, void 0, function* () {
            logger("playTransactions");
            if (tree.key == undefined) {
                throw new Error("playing transactions on a tree requires the tree to have a private key, use tree.key = <ecdsaKey>");
            }
            const tw = yield TupeloWasm.get();
            let transBits = new Array();
            for (var t of transactions) {
                const serialized = t.serializeBinary();
                transBits = transBits.concat(serialized);
            }
            const store = tree.store;
            const privateKey = tree.key.privateKey ? tree.key.privateKey : new Uint8Array();
            if (privateKey.length == 0) {
                throw new Error("can only play transactions on a tree with a private key attached");
            }
            const resp = yield tw.playTransactions({
                notaryGroup: notaryGroup.serializeBinary(),
                publisher: publisher,
                blockService: store,
                privateKey: privateKey,
                tip: tree.tip,
                transactions: transBits,
            });
            const currState = signatures_pb_1.TreeState.deserializeBinary(resp);
            tree.tip = new cids_1.default(Buffer.from(currState.getNewTip_asU8()));
            return currState;
        });
    }
    Tupelo.playTransactions = playTransactions;
})(Tupelo = exports.Tupelo || (exports.Tupelo = {}));
exports.default = Tupelo;
